﻿/********************************************************************************//*!
 * @file  sample_mmu.c
 * @brief  To use HWLib for MMU Management API example program.
 * 
 * @details  Run the API category below.
 *           - MMU Management API
 *               - MMU Management
 *               - MMU Virtual Address Space Creation
 * 
 * @note  nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成. 
 * @date 2014/08/18  ---  -----  v14.0化 (誤植訂正、操作説明の表示を追加). 
 * @date 2015/06/25  ---  #20263  v15.0対応/文字コード変更(UTF-8) 他. 
 * @date 2016/07/26  ---  -----  v16.0での確認およびHelio以外の評価ボードへの対応を実施.
 * @date 2018/07/02  ---  #25940  All in One ベアメタル・アプリケーション v17.1 への対応を実施.
 * @date 2019/01/18  ---  -----   All in One ベアメタル・アプリケーション v18.1 への対応を実施.
 * 
 * @attention 
 * Copyright (C) 2013-2019 MACNICA,Inc. All Rights Reserved.\n
 *   This software is licensed "AS IS". 
 *   Please perform use of this software by a user's own responsibility and expense.
 *   It cannot guarantee in the maker side about the damage which occurred by the ab-
 *   ility not to use or use this software, and all damage that occurred secondarily.
 **//*******************************************************************************/

/***********************************************************************************
 *  includes 
 ***********************************************************************************/
#include <stdio.h>
#include <string.h>
#include "hwlib.h"
#include "socal/socal.h"
#include "socal/hps.h"
#include "alt_generalpurpose_io.h"
#include "alt_globaltmr.h"
#include "alt_clock_manager.h"
#include "alt_interrupt.h"
#include "alt_address_space.h"
#include "alt_mmu.h"

#include "sample_app_setting.h"
#include "util_time_measurement.h"
#include "util_interrupt_log.h"


/***********************************************************************************
 *  externs 
 ***********************************************************************************/

/***********************************************************************************
 *  proto types 
 ***********************************************************************************/
#ifdef soc_cv_av
static void sample_adrspcmng_setting_remap(void);
static void sample_adrspcmng_setting_l2addrfilter(uint32_t addr_filt_start, uint32_t addr_filt_end);
static void sample_adrspcmng_print_ACPIDMapper(void);
#endif /* soc_cv_av */
void* sample_mmu_test_tlb_alloc(const size_t size, void * context);
void sample_mmu_init_MMU(void);
void sample_mmu_test_ttb_desc_set_APIs(void);
void sample_mmu_test_TLB_maintenance_operations_APIs(void);
void sample_mmu_test_print_registers(void);
void sample_mmu_test_register_setget_APIs(void);
void sample_mmu_test_init(void);
void sample_mmu_test_uninit(void);
void sample_mmu_test_print_usage(void);
void sample_mmu_test_main(void);
int  sample_mmu_test_cmd(char* options);
// prototype (Common Functions) //
static void sample_dmac_setting_testdata(uint32_t* start, uint32_t* endnext);

/***********************************************************************************
 *  definitions (define, enum, typedef, etc..) 
 ***********************************************************************************/
/* MMU table buffer size */
#define MMU_TEST_TLB_HEAP_SIZE  (32 * _KB)

/* TLB memory should be aligned 16KB */
#define MMU_TEST_TLB_ALIGNMENT  (16 * _KB)

/******** Definition Memory Region ********/
#define MAX_MEM_REGION_NUM      16

#if APP_LOAD_ADDR==0x00200000 // Program Load Address: 00200000

/*! SDRAM .. PA:0x00000000~0x001FFFFF <--> VA: (=PA) */
#define SDRAM_MMU_PHYS_ADDR			0x00000000
#define SDRAM_MMU_VIRT_ADDR			SDRAM_MMU_PHYS_ADDR
#define SDRAM_MMU_SIZE				(2 * _MB)

/*! SDRAMx(Private Region) .. PA:0x00200000~0x005FFFFF <--> VA: (=PA) */
#define	SDRAMx_MMU_PHYS_ADDR		0x00200000
#define	SDRAMx_MMU_VIRT_ADDR		SDRAMx_MMU_PHYS_ADDR
#define	SDRAMx_MMU_SIZE				(4 * _MB)

/*! SDRAM2 .. PA:0x00800000~0x00BFFFFF <--> VA:0x00600000~0x009FFFFF */
#define SDRAM2_MMU_PHYS_ADDR		0x00800000
#define SDRAM2_MMU_VIRT_ADDR		0x00600000
#define SDRAM2_MMU_SIZE				(4 * _MB)

#else	// APP_LOAD_ADDR!=0x00200000

/*! SDRAM .. PA:0x00000000~0x003FFFFF <--> VA: (=PA) */
#define SDRAM_MMU_PHYS_ADDR			0x00000000
#define SDRAM_MMU_VIRT_ADDR			SDRAM_MMU_PHYS_ADDR
#define SDRAM_MMU_SIZE				(4 * _MB)

/*! SDRAM2 .. PA:0x00800000~0x00BFFFFF <--> VA:0x00400000~0x007FFFFF */
#define SDRAM2_MMU_PHYS_ADDR		0x00800000
#define SDRAM2_MMU_VIRT_ADDR		0x00400000
#define SDRAM2_MMU_SIZE				(4 * _MB)

#endif	// APP_LOAD_ADDR==0x00200000

/*! SDRAM3 .. PA:0x02000000~0x03FFFFFF <--> VA: (=PA) */
#define SDRAM3_MMU_PHYS_ADDR		0x02000000
#define SDRAM3_MMU_VIRT_ADDR		SDRAM3_MMU_PHYS_ADDR
#define SDRAM3_MMU_SIZE				((1024 * _MB) - (32 * _MB))

/*! HPS2FPGA .. PA:0xC000000~0xFBFFFFFF <--> VA: (=PA) */
#define HPS2FPGA_MMU_PHYS_ADDR		0xC0000000
#define HPS2FPGA_MMU_VIRT_ADDR		HPS2FPGA_MMU_PHYS_ADDR
#define HPS2FPGA_MMU_SIZE			(960 * _MB)

/*! LW_HPS2FPGA .. PA:0xFF200000~0xFF3FFFFF <--> VA: (=PA) */
#define LW_HPS2FPGA_MMU_PHYS_ADDR	0xFF200000
#define LW_HPS2FPGA_MMU_VIRT_ADDR	LW_HPS2FPGA_MMU_PHYS_ADDR
#define LW_HPS2FPGA_MMU_SIZE		(2 * _MB)

#ifdef soc_cv_av
/*! PERIPHRALS .. PA:0xFF400000~0xFFEFFFFF <--> VA: (=PA) */
#define PERIPHRALS_MMU_PHYS_ADDR	0xFF400000
#define PERIPHRALS_MMU_VIRT_ADDR	PERIPHRALS_MMU_PHYS_ADDR
#define PERIPHRALS_MMU_SIZE			(11 * _MB)

/*! BOOTROM .. PA:0xFFFD0000~0xFFFDFFFF <--> VA: (=PA) */
#define BOOTROM_MMU_PHYS_ADDR		0xFFFD0000
#define BOOTROM_MMU_VIRT_ADDR		BOOTROM_MMU_PHYS_ADDR
#define BOOTROM_MMU_SIZE			(64 * _KB)

/*! MPUSCU .. PA:0xFFFEC000~0xFFFEDFFF <--> VA: (=PA) */
#define MPUSCU_MMU_PHYS_ADDR		0xFFFEC000
#define MPUSCU_MMU_VIRT_ADDR		MPUSCU_MMU_PHYS_ADDR
#define MPUSCU_MMU_SIZE				(8 * _KB)

/*! MPUL2 .. PA:0xFFFEF000~0xFFFEFFFF <--> VA: (=PA) */
#define MPUL2_MMU_PHYS_ADDR			0xFFFEF000
#define MPUL2_MMU_VIRT_ADDR			MPUL2_MMU_PHYS_ADDR
#define MPUL2_MMU_SIZE				(4 * _KB)

/*! ONCHIPRAM .. PA:0xFFFF0000~0xFFFFFFFF <--> VA: (=PA) */
#define RAM_MMU_PHYS_ADDR			0xFFFF0000
#define RAM_MMU_VIRT_ADDR			RAM_MMU_PHYS_ADDR
#define RAM_MMU_SIZE				(64 * _KB)
#else /* soc_cv_av */
/*! A10 PERIPHRALS .. PA:0xFF400000~0xFFDFFFFF <--> VA: (=PA) */
#define PERIPHRALS_MMU_PHYS_ADDR	0xFF400000
#define PERIPHRALS_MMU_VIRT_ADDR	PERIPHRALS_MMU_PHYS_ADDR
#define PERIPHRALS_MMU_SIZE			(10 * _MB)

/*! A10 ONCHIPRAM .. PA:0xFFE00000~0xFFE3FFFF <--> VA: (=PA) */
#define RAM_MMU_PHYS_ADDR			0xFFE00000
#define RAM_MMU_VIRT_ADDR			RAM_MMU_PHYS_ADDR
#define RAM_MMU_SIZE				(256 * _KB)

/*! A10 BOOTROM .. PA:0xFFFC0000~0xFFFDFFFF <--> VA: (=PA) */
#define BOOTROM_MMU_PHYS_ADDR		0xFFFC0000
#define BOOTROM_MMU_VIRT_ADDR		BOOTROM_MMU_PHYS_ADDR
#define BOOTROM_MMU_SIZE			(128 * _KB)

/*! A10 MPUSCU .. PA:0xFFFFC000~0xFFFFDFFF <--> VA: (=PA) */
#define MPUSCU_MMU_PHYS_ADDR		0xFFFFC000
#define MPUSCU_MMU_VIRT_ADDR		MPUSCU_MMU_PHYS_ADDR
#define MPUSCU_MMU_SIZE				(8 * _KB)

/*! A10 MPUL2 .. PA:0xFFFFF000~0xFFFFFFFF <--> VA: (=PA) */
#define MPUL2_MMU_PHYS_ADDR			0xFFFFF000
#define MPUL2_MMU_VIRT_ADDR			MPUL2_MMU_PHYS_ADDR
#define MPUL2_MMU_SIZE				(4 * _KB)
#endif /* soc_cv_av */


/***********************************************************************************
 *  variables
 ***********************************************************************************/
//typedef struct ALT_MMU_MEM_REGION_s
//{
//  void *                 va;        /*!< The beginning virtual address for the memory
//                                     *   region. The address must be aligned to one of 4KiB,
//                                     *   64KiB, 1MiB, or 16MiB boundaries.
//                                     */
//  void *                 pa;        /*!< The beginning physical address mapping for the
//                                     *   virtual address of the memory region. The address
//                                     *   must be aligned to one of 4KiB, 64KiB, 1MiB, or 16MiB
//                                     *   boundaries.
//                                     */
//  uint32_t               size;      /*!< The size of the memory region in bytes. The size
//                                     *   must be a multiple of 4KiB, 64KiB, 1MiB, or 16MiB
//                                     *   sizes.
//                                     */
//  ALT_MMU_AP_t           access;    /*!< The access permissions for the memory region. */
//  ALT_MMU_ATTR_t         attributes;/*!< The memory region attributes. These attributes
//                                     *   determine the memory type (ordering), cache
//                                     *   policy, and as a possible side effect, the
//                                     *   shareablity of the memory region.
//                                     */
//  ALT_MMU_TTB_S_t        shareable; /*!< The shareability of the memory region. */
//  ALT_MMU_TTB_XN_t       execute;   /*!< Whether instructions can be executed from this
//                                     *   memory region.
//                                     */
//  ALT_MMU_TTB_NS_t       security;  /*!< Controls whether address translations made from
//                                     *   the secure state translate physical address in
//                                     *   the secure or non-secure address map.
//                                     */
//} ALT_MMU_MEM_REGION_t;
#if APP_LOAD_ADDR==0x00200000 // Program Load Address: 00200000

int	ValidMmuMemoryRegion = 11;		/*!< The number of valid descriptors. */
ALT_MMU_MEM_REGION_t MmuMemoryRegion[MAX_MEM_REGION_NUM] = {
	/* [0] SDRAM	*/
	{ (void*)SDRAM_MMU_VIRT_ADDR, (void*)SDRAM_MMU_PHYS_ADDR, SDRAM_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, *//* This value must be match with TTBR0.S if TLB on this area */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [1] SDRAMx	*/
	{ (void*)SDRAMx_MMU_VIRT_ADDR, (void*)SDRAMx_MMU_PHYS_ADDR, SDRAMx_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, *//* This value must be match with TTBR0.S if TLB on this area */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [2] SDRAM2	*/
	{ (void*)SDRAM2_MMU_VIRT_ADDR, (void*)SDRAM2_MMU_PHYS_ADDR, SDRAM2_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, *//* This value must be match with TTBR0.S if TLB on this area */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [3] SDRAM3 (include .text)	*/
	{ (void*)SDRAM3_MMU_VIRT_ADDR, (void*)SDRAM3_MMU_PHYS_ADDR, SDRAM3_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, *//* This value must be match with TTBR0.S if TLB on this area */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [4] HPS2FPGA	*/
	{ (void*)HPS2FPGA_MMU_VIRT_ADDR, (void*)HPS2FPGA_MMU_PHYS_ADDR, HPS2FPGA_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_DEVICE,			/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [5] LW_HPS2FPGA	*/
	{ (void*)LW_HPS2FPGA_MMU_VIRT_ADDR, (void*)LW_HPS2FPGA_MMU_PHYS_ADDR, LW_HPS2FPGA_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_DEVICE,			/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
#ifdef soc_cv_av
	/* [6] PERIPHRALS	*/
	{ (void*)PERIPHRALS_MMU_VIRT_ADDR, (void*)PERIPHRALS_MMU_PHYS_ADDR, PERIPHRALS_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [7] BOOTROM	*/
	{ (void*)BOOTROM_MMU_VIRT_ADDR, (void*)BOOTROM_MMU_PHYS_ADDR, BOOTROM_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_READ_ONLY,			/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [8] MPU_SCU	*/
	{ (void*)MPUSCU_MMU_VIRT_ADDR, (void*)MPUSCU_MMU_PHYS_ADDR, MPUSCU_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [9] MPU_L2	*/
	{ (void*)MPUL2_MMU_VIRT_ADDR, (void*)MPUL2_MMU_PHYS_ADDR, MPUL2_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [10] RAM	*/
	{ (void*)RAM_MMU_VIRT_ADDR, (void*)RAM_MMU_PHYS_ADDR, RAM_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
#else /* soc_cv_av */
	/* [6] A10 PERIPHRALS	*/
	{ (void*)PERIPHRALS_MMU_VIRT_ADDR, (void*)PERIPHRALS_MMU_PHYS_ADDR, PERIPHRALS_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [7] A10 RAM	*/
	{ (void*)RAM_MMU_VIRT_ADDR, (void*)RAM_MMU_PHYS_ADDR, RAM_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [8] A10 BOOTROM	*/
	{ (void*)BOOTROM_MMU_VIRT_ADDR, (void*)BOOTROM_MMU_PHYS_ADDR, BOOTROM_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_READ_ONLY,			/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [9] A10 MPU_SCU	*/
	{ (void*)MPUSCU_MMU_VIRT_ADDR, (void*)MPUSCU_MMU_PHYS_ADDR, MPUSCU_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [10] A10 MPU_L2	*/
	{ (void*)MPUL2_MMU_VIRT_ADDR, (void*)MPUL2_MMU_PHYS_ADDR, MPUL2_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
#endif /* soc_cv_av */
	/* [11] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  ALT_MMU_AP_NO_ACCESS,			/* access, */
	  ALT_MMU_ATTR_FAULT,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [12] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  ALT_MMU_AP_NO_ACCESS,			/* access, */
	  ALT_MMU_ATTR_FAULT,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [13] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  ALT_MMU_AP_NO_ACCESS,			/* access, */
	  ALT_MMU_ATTR_FAULT,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [14] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  ALT_MMU_AP_NO_ACCESS,			/* access, */
	  ALT_MMU_ATTR_FAULT,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [15] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  ALT_MMU_AP_NO_ACCESS,			/* access, */
	  ALT_MMU_ATTR_FAULT,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	}
};

#else	// APP_LOAD_ADDR!=0x00200000

int	ValidMmuMemoryRegion = 10;		/*!< The number of valid descriptors. */
ALT_MMU_MEM_REGION_t MmuMemoryRegion[MAX_MEM_REGION_NUM] = {
	/* [0] SDRAM	*/
	{ (void*)SDRAM_MMU_VIRT_ADDR, (void*)SDRAM_MMU_PHYS_ADDR, SDRAM_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, *//* This value must be match with TTBR0.S if TLB on this area */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [1] SDRAM2	*/
	{ (void*)SDRAM2_MMU_VIRT_ADDR, (void*)SDRAM2_MMU_PHYS_ADDR, SDRAM2_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, *//* This value must be match with TTBR0.S if TLB on this area */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [2] SDRAM3 (include .text)	*/
	{ (void*)SDRAM3_MMU_VIRT_ADDR, (void*)SDRAM3_MMU_PHYS_ADDR, SDRAM3_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, *//* This value must be match with TTBR0.S if TLB on this area */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [3] HPS2FPGA	*/
	{ (void*)HPS2FPGA_MMU_VIRT_ADDR, (void*)HPS2FPGA_MMU_PHYS_ADDR, HPS2FPGA_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_DEVICE,			/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [4] LW_HPS2FPGA	*/
	{ (void*)LW_HPS2FPGA_MMU_VIRT_ADDR, (void*)LW_HPS2FPGA_MMU_PHYS_ADDR, LW_HPS2FPGA_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_DEVICE,			/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
#ifdef soc_cv_av
	/* [5] PERIPHRALS	*/
	{ (void*)PERIPHRALS_MMU_VIRT_ADDR, (void*)PERIPHRALS_MMU_PHYS_ADDR, PERIPHRALS_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [6] BOOTROM	*/
	{ (void*)BOOTROM_MMU_VIRT_ADDR, (void*)BOOTROM_MMU_PHYS_ADDR, BOOTROM_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_READ_ONLY,			/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [7] MPU_SCU	*/
	{ (void*)MPUSCU_MMU_VIRT_ADDR, (void*)MPUSCU_MMU_PHYS_ADDR, MPUSCU_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [8] MPU_L2	*/
	{ (void*)MPUL2_MMU_VIRT_ADDR, (void*)MPUL2_MMU_PHYS_ADDR, MPUL2_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [9] RAM	*/
	{ (void*)RAM_MMU_VIRT_ADDR, (void*)RAM_MMU_PHYS_ADDR, RAM_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
#else /* soc_cv_av */
	/* [5] A10 PERIPHRALS	*/
	{ (void*)PERIPHRALS_MMU_VIRT_ADDR, (void*)PERIPHRALS_MMU_PHYS_ADDR, PERIPHRALS_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [6] A10 RAM	*/
	{ (void*)RAM_MMU_VIRT_ADDR, (void*)RAM_MMU_PHYS_ADDR, RAM_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [7] A10 BOOTROM	*/
	{ (void*)BOOTROM_MMU_VIRT_ADDR, (void*)BOOTROM_MMU_PHYS_ADDR, BOOTROM_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_READ_ONLY,			/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [8] A10 MPU_SCU	*/
	{ (void*)MPUSCU_MMU_VIRT_ADDR, (void*)MPUSCU_MMU_PHYS_ADDR, MPUSCU_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [9] A10 MPU_L2	*/
	{ (void*)MPUL2_MMU_VIRT_ADDR, (void*)MPUL2_MMU_PHYS_ADDR, MPUL2_MMU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
#endif /* soc_cv_av */
	/* [10] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  ALT_MMU_AP_NO_ACCESS,			/* access, */
	  ALT_MMU_ATTR_FAULT,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [11] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  ALT_MMU_AP_NO_ACCESS,			/* access, */
	  ALT_MMU_ATTR_FAULT,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [12] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  ALT_MMU_AP_NO_ACCESS,			/* access, */
	  ALT_MMU_ATTR_FAULT,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [13] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  ALT_MMU_AP_NO_ACCESS,			/* access, */
	  ALT_MMU_ATTR_FAULT,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [14] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  ALT_MMU_AP_NO_ACCESS,			/* access, */
	  ALT_MMU_ATTR_FAULT,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [15] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  ALT_MMU_AP_NO_ACCESS,			/* access, */
	  ALT_MMU_ATTR_FAULT,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	}
};

#endif	// APP_LOAD_ADDR==0x00200000

static uint32_t mmu_test_tlb_heap[MMU_TEST_TLB_HEAP_SIZE] __attribute__ ((aligned MMU_TEST_TLB_ALIGNMENT)) ;
uint32_t *mmu_test_tlb_l1 = NULL;

const char* PageAddressString[4] = {
	"Page 0 - MPU address range 0x00000000 - 0x3FFFFFFF",
	"Page 1 - MPU address range 0x40000000 - 0x7FFFFFFF",
	"Page 2 - MPU address range 0x80000000 - 0xBFFFFFFF",
	"Page 3 - MPU address range 0xC0000000 - 0xFFFFFFFF"
};

/***********************************************************************************
 *  functions 
 ***********************************************************************************/

#ifdef soc_cv_av
/********************************************************************************//*!
 * @fn  static void sample_adrspcmng_setting_remap(void)
 * @brief  Setting for the Address Space Remap Register.
 *
 * @details  Run the following APIs.
 *           - alt_addr_space_remap
 *           - alt_mpu_addr_space_remap_0_to_sdram
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/13  ---  #13144  新規作成.
 **//*******************************************************************************/
static void sample_adrspcmng_setting_remap(void)
{
	ALT_STATUS_CODE result_code;
	// Remap Setting Value 
	ALT_ADDR_SPACE_MPU_ATTR_t			mpu_attr = ALT_ADDR_SPACE_MPU_ZERO_AT_OCRAM;
	ALT_ADDR_SPACE_NONMPU_ATTR_t		nonmpu_attr = ALT_ADDR_SPACE_NONMPU_ZERO_AT_SDRAM;
	ALT_ADDR_SPACE_H2F_BRIDGE_ATTR_t	h2f_attr = ALT_ADDR_SPACE_H2F_ACCESSIBLE;
	ALT_ADDR_SPACE_LWH2F_BRIDGE_ATTR_t	lwh2f_attr = ALT_ADDR_SPACE_LWH2F_ACCESSIBLE;
	
	/* -- Address Space Mapping Control -- */
	/* Usage:	ALT_STATUS_CODE  alt_addr_space_remap (ALT_ADDR_SPACE_MPU_ATTR_t mpu_attr, ALT_ADDR_SPACE_NONMPU_ATTR_t nonmpu_attr, ALT_ADDR_SPACE_H2F_BRIDGE_ATTR_t h2f_attr, ALT_ADDR_SPACE_LWH2F_BRIDGE_ATTR_t lwh2f_attr);	*/
	/* Usage:	ALT_STATUS_CODE  alt_mpu_addr_space_remap_0_to_sdram (void);	*/
	printf("==== HWLib test : Execute the Address Space Remap Setting ====\n");

	result_code = alt_addr_space_remap(mpu_attr, nonmpu_attr, h2f_attr, lwh2f_attr);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_addr_space_remap() .... result=%d\n", (int)result_code);
	
	result_code = alt_mpu_addr_space_remap_0_to_sdram();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mpu_addr_space_remap_0_to_sdram() .... result=%d\n", (int)result_code);

	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_adrspcmng_setting_l2addrfilter(uint32_t addr_filt_start, uint32_t addr_filt_end)
 * @brief  Setting for the L2 Cache Address Filter.
 *
 * @details  Run the following APIs.
 *           - alt_l2_addr_filter_cfg_get
 *           - alt_l2_addr_filter_cfg_set
 * @param[in] addr_filt_start  Filter Start Address.
 * @param[in] addr_filt_end  Filter End Address.
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/13  ---  #13144  新規作成.
 **//*******************************************************************************/
static void sample_adrspcmng_setting_l2addrfilter(uint32_t addr_filt_start, uint32_t addr_filt_end)
{
	ALT_STATUS_CODE result_code;
	uint32_t	before_filt_start;
	uint32_t	before_filt_end;
	
	/* -- L2 Cache Address Filter -- */
	/* Usage:	ALT_STATUS_CODE  alt_l2_addr_filter_cfg_get (uint32_t *addr_filt_start, uint32_t *addr_filt_end);	*/
	/* Usage:	ALT_STATUS_CODE  alt_l2_addr_filter_cfg_set (uint32_t addr_filt_start, uint32_t addr_filt_end);	*/
	printf("==== HWLib test : Execute the L2 Cache Address Filter Setting ====\n");
	result_code = alt_l2_addr_filter_cfg_get(&before_filt_start, &before_filt_end);
	printf("<<before setting>>\n");
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_l2_addr_filter_cfg_get() .... result=%d\n", (int)result_code);
	printf(" +-- addr_filt_start = 0x%08X\n", (int)before_filt_start);
	printf(" +-- addr_filt_end   = 0x%08X\n", (int)before_filt_end);
	
	result_code = alt_l2_addr_filter_cfg_set(addr_filt_start, addr_filt_end);
	printf("<<after setting>>\n");
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_l2_addr_filter_cfg_set() .... result=%d\n", (int)result_code);
	printf(" +-- addr_filt_start = 0x%08X\n", (int)addr_filt_start);
	printf(" +-- addr_filt_end   = 0x%08X\n", (int)addr_filt_end);
	
	printf("\n");
	
	return;
}

/********************************************************************************//*!
 * @fn  static void sample_adrspcmng_print_ACPIDMapper(void)
 * @brief  Print Current Settings of ACP ID Mapper.
 *
 * @details  Run the following APIs.
 *           - alt_acpid_map_read_options_get
 *           - alt_acpid_map_write_options_get
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  Target APIs are not implemented in the current version.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/13  ---  #13144  新規作成.
 **//*******************************************************************************/
static void sample_adrspcmng_print_ACPIDMapper(void)
{
#if 0
	ALT_STATUS_CODE result_code;
	int i;
	bool fixed;
	uint32_t input_id;
	ALT_ACP_ID_MAP_PAGE_t page;
	uint32_t aruser;

	/* -- ACP Memory Coherence and ID Mapping -- */
	/* Usage:	ALT_STATUS_CODE  alt_acpid_map_read_options_get (const uint32_t output_id, bool *fixed, uint32_t *input_id, ALT_ACP_ID_MAP_PAGE_t *page, uint32_t *aruser);	*/
	/* Usage:	ALT_STATUS_CODE  alt_acpid_map_write_options_get (const uint32_t output_id, bool *fixed, uint32_t *input_id, ALT_ACP_ID_MAP_PAGE_t *page, uint32_t *awuser);	*/
	printf("==== HWLib test : Print Current Setting of ACP ID Mapper ====\n");
	printf("<<Read Transactions>>\n");
	for(i=0; i<=7; i++)
	{
		result_code = alt_acpid_map_read_options_get(i, &fixed, &input_id, &page, &aruser);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("[%d] alt_acpid_map_read_options_get() .... result=%d\n", i, (int)result_code);
		printf(" +-- output_id = %d\n", i);
		printf(" +-- fixed = %s\n", ((fixed == true)?"true":"false"));
		printf(" +-- input_id = 0x%08X (%d)\n", (int)input_id, (int)input_id);
		printf(" +-- page = %s\n", PageAddressString[page]);
		printf(" +-- aruser = 0x%08X (%d)\n", (int)aruser, (int)aruser);
		printf("\n");
	}

	printf("<<Write Transactions>>\n");
	for(i=0; i<=7; i++)
	{
		result_code = alt_acpid_map_write_options_get(i, &fixed, &input_id, &page, &aruser);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("[%d] alt_acpid_map_write_options_get() .... result=%d\n", i, (int)result_code);
		printf(" +-- output_id = %d\n", i);
		printf(" +-- fixed = %s\n", ((fixed == true)?"true":"false"));
		printf(" +-- input_id = 0x%08X (%d)\n", (int)input_id, (int)input_id);
		printf(" +-- page = %s\n", PageAddressString[page]);
		printf(" +-- aruser = 0x%08X (%d)\n", (int)aruser, (int)aruser);
		printf("\n");
	}

	printf("\n");
#endif
	return;
}
#endif /* soc_cv_av */

/********************************************************************************//*!
 * @fn  void* sample_mmu_test_tlb_alloc(const size_t size, void * context)
 * @brief  Memory allocation for 1st and 2nd translation tables.
 *
 * @details  User defined function used for allocating storage for first and second level translation tables.
 *           This is the function to specify the 4th argument of the API 'alt_mmu_va_space_create'.
 * @param[in] size  The size in bytes of the storage request. The argument will be either the value ALT_MMU_TTB1_SIZE or ALT_MMU_TTB2_SIZE.
 * @param[in] context  A user provided context for the allocator function.
 * @retval void*  A sixteen-word aligned pointer to the allocated memory or NULL if the storage request cannot be satisfied. 
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成.
 **//*******************************************************************************/
void* sample_mmu_test_tlb_alloc(const size_t size, void * context)
{
	void* p = NULL;
	printf("Execute ttb_alloc Function.\n");

	/* User defined function used for allocating storage for first and second level translation tables. */

	if(size >= sizeof(mmu_test_tlb_heap)){
		printf("ERROR!!: mmu_tlb_alloc() 1st argument ""size"" is too large!\n");
	} else if(((uint32_t)context % MMU_TEST_TLB_ALIGNMENT) != 0){
		printf("ERROR!!: mmu_tlb_alloc() 2st argument ""context"" is not aligned!\n");
	} else {
		p = context;
	}

	return p ;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_init_MMU(void)
 * @brief  Do the initial settings of the MMU.
 *
 * @details  Run the following APIs.
 *           - alt_mmu_disable
 *           - alt_mmu_va_space_create
 *           - alt_mmu_va_space_enable
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成.
 **//*******************************************************************************/
void sample_mmu_init_MMU(void)
{
	ALT_STATUS_CODE result_code;

	/* disable MMU */
	alt_mmu_disable();

	/* create MMU tables */
	/* Usage:	ALT_STATUS_CODE  alt_mmu_va_space_create (uint32_t **ttb1, const ALT_MMU_MEM_REGION_t *mem_regions, const size_t num_mem_regions, alt_mmu_ttb_alloc_t ttb_alloc, void *ttb_alloc_context);	*/
	result_code = alt_mmu_va_space_create(&mmu_test_tlb_l1, MmuMemoryRegion, ValidMmuMemoryRegion, sample_mmu_test_tlb_alloc, (void*)mmu_test_tlb_heap);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_va_space_create() .... result=%d\n", (int)result_code);

	if(result_code == ALT_E_SUCCESS){
		/* enable MMU */
		/* Usage:	ALT_STATUS_CODE  alt_mmu_va_space_enable (const uint32_t *ttb1);	*/
		result_code = alt_mmu_va_space_enable(mmu_test_tlb_l1);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("alt_mmu_va_space_enable() .... result=%d\n", (int)result_code);
	}

	if(result_code != ALT_E_SUCCESS){
		printf("MMU Init error\n" );
	}

	return;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_test_ttb_desc_set_APIs(void)
 * @brief  Test the API for setting transration table descriptor.
 *
 * @details  Run the following APIs.
 *           - alt_mmu_ttb1_desc_set
 *           - alt_mmu_ttb2_desc_set
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成.
 **//*******************************************************************************/
void sample_mmu_test_ttb_desc_set_APIs(void)
{
	ALT_STATUS_CODE result_code;
	uint32_t* discp_ttb1;
	uint32_t* discp_ttb2;
	uint32_t discv;
	uint32_t target_addr;
	int i;

	/* disable MMU */
	alt_mmu_disable();

	/* Usage:	ALT_STATUS_CODE  alt_mmu_ttb1_desc_set (uint32_t *ttb1, const void *va, const uint32_t desc);	*/
	/* -- For the tests to customize TTB1. -- */
	/* original : SDRAM3 .. PA:0x02000000~0x03FFFFFF <--> VA:0x02000000~0x03FFFFFF */
	/* <After customization> */
	/* PA:0x02000000~0x03FFFFFF <--> VA:0x02000000~0x03FFFFFF */
	/* PA:0x03000000~0x03FFFFFF <--> VA:0x04000000~0x04FFFFFF */
	/* VA:0x03000000~0x03FFFFFF is not allocated. */
	discp_ttb1 = (uint32_t*)alt_mmu_TTBR0_get();
	target_addr = 0x03000000;
	discv = discp_ttb1[(target_addr / (1 * _MB))];	/* Get discriptor value used for VA:0x03000000 */
	for(i = 0; i < 16; i++){
		/* VA:0x03000000~0x03FFFFFF is not allocated. */
		result_code = alt_mmu_ttb1_desc_set(discp_ttb1, (void*)(target_addr + (i * (1 * _MB))), 0);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("alt_mmu_ttb1_desc_set(VA:0x%08X,0) .... result=%d\n", (int)target_addr, (int)result_code);
	}
	/* PA:0x03000000~0x03FFFFFF <--> VA:0x04000000~0x04FFFFFF */
	result_code = alt_mmu_ttb1_desc_set(discp_ttb1, (void*)(0x01000000 + target_addr), discv);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_ttb1_desc_set(VA:0x%08X,0x%08X) .... result=%d\n", (int)(0x01000000 + target_addr), (int)discv, (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_ttb2_desc_set (const uint32_t *ttb1, const void *va, const uint32_t desc);	*/
	/* -- For the tests to customize TTB2. -- */
	/* original :		   PA:0xFFF00000~0xFFFCFFFF <--> VA: not allocated        */
	/*		  ONCHIPRAM .. PA:0xFFFF0000~0xFFFFFFFF <--> VA:0xFFFF0000~0xFFFFFFFF */
	/* <After customization> */
	/*		  ONCHIPRAM .. PA:0xFFFF0000~0xFFFFFFFF <--> VA:0xFFF00000~0xFFF0FFFF */
	/* 		               VA:0xFFFF0000~0xFFFFFFFF is not allocated. */
	discv = discp_ttb1[(0xFFF00000 / (1 * _MB))];
	discp_ttb2 = (uint32_t*)(discv & ALT_MMU_TTB1_PAGE_TBL_BASE_ADDR_MASK);
	discv = discp_ttb2[((0xFFFFF000 & ((1 * _MB) - 1)) / (4 * _KB))];	/* Get discriptor value used for VA:0xFFFFF000 */

	//discv &= ALT_MMU_TTB2_SMALL_PAGE_BASE_ADDR_MASK;
	//discv |= ALT_MMU_TTB2_SMALL_PAGE_BASE_ADDR_SET(0xFFFFF000);	/* Set discriptor value to PA:0xFFFFF000 */

	/* VA:0xFFFF0000~0xFFFFFFFF is not allocated. */
	for(i = 0; i < 16; i++){
		result_code = alt_mmu_ttb2_desc_set(discp_ttb1, (void*)(0xFFFF0000 + (i * 0x1000)), 0);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("alt_mmu_ttb2_desc_set(VA:0x%08X,0) .... result=%d\n", (int)(0xFFFF0000 + (i * 0x1000)), (int)result_code);
	}
	/* PA:0xFFFF0000~0xFFFFFFFF <--> VA:0xFFF00000~0xFFF0FFFF */
	result_code = alt_mmu_ttb2_desc_set(discp_ttb1, (void*)0xFFF00000, discv);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_ttb2_desc_set(VA:0xFFF00000,0x%08X) .... result=%d\n", (int)discv, (int)result_code);

	/* enable MMU */
	alt_mmu_enable() ;

	return;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_test_TLB_maintenance_operations_APIs(void)
 * @brief  Test the API for TLB maintenance operation.
 *
 * @details  Run the following APIs.
 *           - alt_mmu_tlb_invalidate
 *           - alt_mmu_tlb_invalidate_is
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成.
 **//*******************************************************************************/
void sample_mmu_test_TLB_maintenance_operations_APIs(void)
{
	ALT_STATUS_CODE result_code;

	/* Usage:	ALT_STATUS_CODE  alt_mmu_tlb_invalidate (void);	*/
	result_code = alt_mmu_tlb_invalidate();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_tlb_invalidate() .... result=%d\n", (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_tlb_invalidate_is (void);	*/
	result_code = alt_mmu_tlb_invalidate_is();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_tlb_invalidate_is() .... result=%d\n", (int)result_code);


	return;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_test_print_registers(void)
 * @brief  Print the MMU related Registers.
 *
 * @details  Print the register values obtained using the API and coprocessor instructions.
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成.
 **//*******************************************************************************/
void sample_mmu_test_print_registers(void)
{
	/* The value returned by the following API doesn't register values. */
	/* Retrieve only the value of the base address of the translation table. */

	/* Usage:	void *  alt_mmu_TTBR0_get (void);	*/
	printf("-- Translation Table Base Register 0 (TTBR0) --\n");
	printf("alt_mmu_TTBR0_get() .... result=0x%08X\n", (int)alt_mmu_TTBR0_get());
	/* Usage:	void *  alt_mmu_TTBR1_get (void);	*/
	printf("-- Translation Table Base Register 1 (TTBR1) --\n");
	printf("alt_mmu_TTBR1_get() .... result=0x%08X\n", (int)alt_mmu_TTBR1_get());
	
	/* API to get the following register values not provided in HWLib. */
	uint32_t ttbr0;			/* Translation Table Base Register 0 (TTBR0) */
	uint32_t ttbr1;			/* Translation Table Base Register 1 (TTBR1) */
	uint32_t ttbcr;			/* Translation Table Base Control Register (TTBCR) */
	uint32_t dacr;			/* Domain Access Control Register (DACR) */
	uint32_t contextidr;	/* Context ID Register (CONTEXTIDR) */
#ifdef GNU_COMPILER
	__asm("MRC p15, 0, %0, c2, c0, 0" : "=r" (ttbr0));
	__asm("MRC p15, 0, %0, c2, c0, 1" : "=r" (ttbr1));
	__asm("MRC p15, 0, %0, c2, c0, 2" : "=r" (ttbcr));
	__asm("MRC p15, 0, %0, c3, c0, 0" : "=r" (dacr));
	__asm("MRC p15, 0, %0, c13, c0, 1" : "=r" (contextidr));
#endif
#ifdef ARM_COMPILER
	__asm("MRC p15, 0, ttbr0, c2, c0, 0");
	__asm("MRC p15, 0, ttbr1, c2, c0, 1");
	__asm("MRC p15, 0, ttbcr, c2, c0, 2");
	__asm("MRC p15, 0, dacr, c3, c0, 0");
	__asm("MRC p15, 0, contextidr, c13, c0, 1");
#endif
	printf("-- TTBRx real values, and the other register values. --\n");
	printf("TTBR0      = 0x%08X\n", (int)ttbr0);
	printf("TTBR1      = 0x%08X\n", (int)ttbr1);
	printf("TTBCR      = 0x%08X\n", (int)ttbcr);
	printf("DACR       = 0x%08X\n", (int)dacr);
	printf("CONTEXTIDR = 0x%08X\n", (int)contextidr);

	return;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_test_register_setget_APIs(void)
 * @brief  Test the API to read and write MMU related register.
 *
 * @details  Test the API name corresponding to the following rules.
 *           - alt_mmu_XXXX_get
 *           - alt_mmu_XXXX_set
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成.
 **//*******************************************************************************/
void sample_mmu_test_register_setget_APIs(void)
{
	ALT_STATUS_CODE result_code;
	uint32_t setval_u32, setval_u32_2;
	static ALT_MMU_DAP_t test_domain_ap[16] = {
		ALT_MMU_DAP_MANAGER,	ALT_MMU_DAP_MANAGER,	ALT_MMU_DAP_MANAGER,	ALT_MMU_DAP_MANAGER,
		ALT_MMU_DAP_CLIENT,		ALT_MMU_DAP_CLIENT,		ALT_MMU_DAP_CLIENT,		ALT_MMU_DAP_CLIENT,
		ALT_MMU_DAP_NO_ACCESS,	ALT_MMU_DAP_NO_ACCESS,	ALT_MMU_DAP_NO_ACCESS,	ALT_MMU_DAP_NO_ACCESS,
		ALT_MMU_DAP_MANAGER,	ALT_MMU_DAP_MANAGER,	ALT_MMU_DAP_MANAGER,	ALT_MMU_DAP_MANAGER
	};

	/* Testing is intended. Run the API sets the value of a register. */
	/* This function should do before you start the MMU setting. */

	/* Usage:	ALT_STATUS_CODE  alt_mmu_TTBR0_set (const void *addr);	*/
	setval_u32 = 0x03000000;
	result_code = alt_mmu_TTBR0_set((void*)setval_u32);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_TTBR0_set(0x%08X) .... result=%d\n", (int)setval_u32, (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_TTBR1_set (const void *addr);	*/
	setval_u32 = 0x04000000;
	result_code = alt_mmu_TTBR1_set((void*)setval_u32);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_TTBR1_set(0x%08X) .... result=%d\n", (int)setval_u32, (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_TTBCR_set (const bool enable_ttbr0_walk, const bool enable_ttbr1_walk, const uint32_t base_addr_width);	*/
	result_code = alt_mmu_TTBCR_set(false, false, 4);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_TTBCR_set(false,false,3) .... result=%d\n", (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_DACR_set (const ALT_MMU_DAP_t domain_ap[], const size_t num_elem);	*/
	result_code = alt_mmu_DACR_set(test_domain_ap, 16);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_DACR_set(test_domain_ap,16) .... result=%d\n", (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_CONTEXTIDR_set (const uint32_t procid, const uint32_t asid);	*/
	setval_u32 = 0;
	setval_u32_2 = 0;
	result_code = alt_mmu_CONTEXTIDR_set(setval_u32, setval_u32_2);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_CONTEXTIDR_set(%d,%d) .... result=%d\n", (int)setval_u32, (int)setval_u32_2, (int)result_code);

	sample_mmu_test_print_registers();
	
	return;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_test_init(void) 
 * @brief  Initializing the sample program. 
 * 
 * @details  Call the initialization function for MMU Management API. 
 *           Other, do the following.
 *           -# Configure the Helio Board GPIO settings. 
 *           -# Print Current Settings of ACP ID Mapper. 
 *           -# Test for set/get APIs.
 *           -# API Test for 'alt_mmu_va_space_storage_required'.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成. 
 **//*******************************************************************************/
void sample_mmu_test_init(void)
{
	ALT_STATUS_CODE result_code;
	size_t trans_table_bytes;
	
	util_intlog_init();
#if 0	// main() 内で cpu0_init() 実施済み
	cpu0_init();
#endif

#ifdef soc_cv_av
	/* Print Current Settings of ACP ID Mapper. */
	sample_adrspcmng_print_ACPIDMapper();
#endif	/* soc_cv_av */
	
	/* Usage:	ALT_STATUS_CODE  alt_mmu_init (void);	*/
	result_code = alt_mmu_init();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_init() .... result=%d\n", (int)result_code);
	
	/* Test for set/get APIs. */
	sample_mmu_test_register_setget_APIs();

	/* Usage:	size_t  alt_mmu_va_space_storage_required (const ALT_MMU_MEM_REGION_t *mem_regions, const size_t num_mem_regions);	*/
	/****************************************************************/
	/* target discriptor : ALT_MMU_MEM_REGION_t MmuMemoryRegion[] 	*/
	/* 					MmuMemoryRegion[0] .. SDRAM					*/
	/* 					MmuMemoryRegion[1] .. SDRAM2				*/
	/* 					MmuMemoryRegion[2] .. SDRAM3 (include .text)*/
	/* 					MmuMemoryRegion[3] .. HPS2FPGA				*/
	/* 					MmuMemoryRegion[4] .. LW_HPS2FPGA			*/
	/* 					MmuMemoryRegion[5] .. PERIPHRALS			*/
	/* 					MmuMemoryRegion[6] .. BOOTROM				*/
	/* 					MmuMemoryRegion[7] .. MPU_SCU				*/
	/* 					MmuMemoryRegion[8] .. MPU_L2				*/
	/* 					MmuMemoryRegion[9] .. RAM					*/
	/****************************************************************/
	trans_table_bytes = alt_mmu_va_space_storage_required(MmuMemoryRegion, 10);
	printf("alt_mmu_va_space_storage_required() ... result=0x%08X(%d) bytes\n", 
		(int)trans_table_bytes, (int)trans_table_bytes);
	
	
	return;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_test_uninit(void) 
 * @brief  Uninitializing the sample program. 
 * 
 * @details  Call the uninitialization function for MMU Management API.
 *           Other, do the following.
 *           -# API Test for 'alt_mmu_ttb1_init'. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成. 
 **//*******************************************************************************/
void sample_mmu_test_uninit(void)
{
	ALT_STATUS_CODE result_code;
	uint32_t ttb1_base_addr;
	
	/* Usage:	ALT_STATUS_CODE  alt_mmu_ttb1_init (uint32_t *ttb1);	*/
	ttb1_base_addr = (uint32_t)alt_mmu_TTBR0_get();
	result_code = alt_mmu_ttb1_init((uint32_t*)ttb1_base_addr);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_ttb1_init(0x%08X) .... result=%d\n", (int)ttb1_base_addr, (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_uninit (void);	*/
	result_code = alt_mmu_uninit();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_uninit() .... result=%d\n", (int)result_code);
	
	return;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_test_print_usage(void) 
 * @brief  Display usage of the sample program. 
 * 
 * @details  Following information displays on the console.
 *            - Overview of HPS for push button functions (or operations)
 *            - Overview of HPS for DIP switch functions (or operations)
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/08/18  ---  #xxxxx  新規作成. 
 **//*******************************************************************************/
void sample_mmu_test_print_usage(void)
{
	printf("\n");
	printf("+--- Usage ( Function of PUSH SW and SLIDE SW )  ---------------+\n");
	printf("+ PUSH SW #0 .... Exit Test loop!!!\n");
	printf("+ PUSH SW #1 .... Run the APIs for setting TTBs as testing.\n");
	printf("+ PUSH SW #2 .... Run the APIs for TLB maintenace as testing.\n");
	printf("+ PUSH SW #3 .... Display the MMU registers.\n");
	printf("+ SLIDE SW#3 .... Reconfigure the MMU at ON/OFF changing.\n");
	printf("+ SLIDE SW#4 .... Select MMU Disable(OFF) or Enable(ON).\n");
	printf("+---------------------------------------------------------------+\n\n");
	
#if 0
	ループ中にHPS用PUSHSWの操作を検出した場合、それぞれ以下の処理を実行します。
		- PUSH SW #0 .... ループを抜けてプログラム終了します。(uninit処理を実行)
		- PUSH SW #1 .... Translation Tableのディスクリプタ書換え用APIを試行。
		- PUSH SW #2 .... TLBのメンテナンス操作用APIを試行。
		- PUSH SW #3 .... MMU関連レジスタの現在値を表示。
		- SLIDE SW#3 .... ON/OFF切替：MMU再初期化（Translation Tableを再作成しMMU有効化）
		- SLIDE SW#4 .... OFF切替：MMU無効化(Disable)、ON切替：MMU有効化(Enable)
#endif
	
	return;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_test_main(void) 
 * @brief  Main routine of the sample program. 
 * 
 * @details  Infinite loop structure works. Make the detection of various types 
 *           of switches in the loop, it performs in various testing processes.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成. 
 **//*******************************************************************************/
void sample_mmu_test_main(void)
{
	uint32_t switch_raw = 0;
	uint32_t switch_raw_bk = 0;
	uint32_t switch_detect = 0;
	uint32_t switch_detect_on = 0;
	uint32_t switch_number = 0;
	bool disp_usage = true;
	
	/* ################################################ */
	/* ## Implement the test setting process here!!! ## */
	/* ################################################ */

#ifdef soc_cv_av
	/* Setting for the Address Space Remap Register. */
	sample_adrspcmng_setting_remap();
	/* Setting for the L2 Cache Address Filter. */
	sample_adrspcmng_setting_l2addrfilter(0, 0xC0000000);
#endif /* soc_cv_av */

	// Setting TEST Data!!! //
	/*! SDRAM .. PA:0x00000000~0x003FFFFF <--> VA:0x00000000~0x003FFFFF */
	/*! SDRAM2 .. PA:0x00800000~0x00BFFFFF <--> VA:0x00400000~0x007FFFFF */
	printf("Setting TEST Data : 0x00000000~0x0000FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0, (uint32_t*)0x0000FFFF);
#if !(APP_LOAD_ADDR==0x00200000) // Program Load Address: 00200000
	printf("Setting TEST Data : 0x00200000~0x0020FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00200000, (uint32_t*)0x0020FFFF);
	printf("Setting TEST Data : 0x002F0000~0x002FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x002F0000, (uint32_t*)0x002FFFFF);
	printf("Setting TEST Data : 0x00300000~0x0030FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00300000, (uint32_t*)0x0030FFFF);
	printf("Setting TEST Data : 0x003F0000~0x003FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x003F0000, (uint32_t*)0x003FFFFF);
	printf("Setting TEST Data : 0x00400000~0x0040FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00400000, (uint32_t*)0x0040FFFF);
	printf("Setting TEST Data : 0x004F0000~0x004FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x004F0000, (uint32_t*)0x004FFFFF);
	printf("Setting TEST Data : 0x00500000~0x0050FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00500000, (uint32_t*)0x0050FFFF);
	printf("Setting TEST Data : 0x005F0000~0x005FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x005F0000, (uint32_t*)0x005FFFFF);
#endif
	printf("Setting TEST Data : 0x00600000~0x0060FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00600000, (uint32_t*)0x0060FFFF);
	printf("Setting TEST Data : 0x006F0000~0x006FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x006F0000, (uint32_t*)0x006FFFFF);
	printf("Setting TEST Data : 0x00700000~0x0070FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00700000, (uint32_t*)0x0070FFFF);
	printf("Setting TEST Data : 0x007F0000~0x007FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x007F0000, (uint32_t*)0x007FFFFF);
	printf("Setting TEST Data : 0x00800000~0x0080FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00800000, (uint32_t*)0x0080FFFF);
	printf("Setting TEST Data : 0x008F0000~0x008FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x008F0000, (uint32_t*)0x008FFFFF);
	printf("Setting TEST Data : 0x00900000~0x0090FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00900000, (uint32_t*)0x0090FFFF);
	printf("Setting TEST Data : 0x009F0000~0x009FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x009F0000, (uint32_t*)0x009FFFFF);
	printf("Setting TEST Data : 0x00A00000~0x00A0FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00A00000, (uint32_t*)0x00A0FFFF);
	printf("Setting TEST Data : 0x00AF0000~0x00AFFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00AF0000, (uint32_t*)0x00AFFFFF);
	printf("Setting TEST Data : 0x00B00000~0x00B0FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00B00000, (uint32_t*)0x00B0FFFF);
	printf("Setting TEST Data : 0x00BF0000~0x00BFFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00BF0000, (uint32_t*)0x00BFFFFF);
	printf("Setting TEST Data : 0x02000000~0x0200FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x02000000, (uint32_t*)0x0200FFFF);
	printf("Setting TEST Data : 0x03000000~0x0300FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x03000000, (uint32_t*)0x0300FFFF);
	printf("Setting TEST Data : 0x03FF0000~0x03FFFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x03FF0000, (uint32_t*)0x03FFFFFF);
	printf("Setting TEST Data : 0xFFFF0000~0xFFFFFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0xFFFF0000, (uint32_t*)0xFFFFFFFF);

	printf("MMU Virtual Address Setting & Enable.\n");
	sample_mmu_init_MMU();
	sample_mmu_test_print_registers();

	printf("==== Start While(1) loop process!!! ============\n");
	switch_raw_bk = sample_detect_switch();
	while(1)
	{
		if(disp_usage)
		{
			sample_mmu_test_print_usage();
			disp_usage = false;
		}

		// ---- Check the Slide-Switch and Push-Switch. ---- 
		switch_raw = sample_detect_switch();
		switch_detect = switch_raw ^ switch_raw_bk;
		switch_detect_on  |= switch_detect & switch_raw;
		
		// Slide-Switch 
		if(switch_detect & SAMPLE_SWITCH_BIT_SLIDEALL)
		{
			switch_number = switch_raw & SAMPLE_SWITCH_BIT_NUM;
			switch_number *= 1;		// To avoid warnings.

			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			if(switch_detect & SAMPLE_SWITCH_BIT_SLIDE0){
				//printf("SAMPLE_SWITCH_BIT_SLIDE0\n");
			}
			if(switch_detect & SAMPLE_SWITCH_BIT_SLIDE1){
				//printf("SAMPLE_SWITCH_BIT_SLIDE1\n");
			}
			if(switch_detect & SAMPLE_SWITCH_BIT_SLIDE2){
				//printf("SAMPLE_SWITCH_BIT_SLIDE2\n");
				printf("MMU Virtual Address Re-Setting & Enable.\n");
				sample_mmu_init_MMU();
			}
			if(switch_detect & SAMPLE_SWITCH_BIT_SLIDE3){
				//printf("SAMPLE_SWITCH_BIT_SLIDE3\n");
				if(switch_raw & SAMPLE_SWITCH_BIT_SLIDE3){
					printf("MMU Enable.\n");
					alt_mmu_enable();
				} else {
					printf("MMU Disable.\n");
					alt_mmu_disable();
				}
			}
			disp_usage = false;
		}
		
		
		// Push-Switch 0
		if( (switch_detect_on & SAMPLE_SWITCH_BIT_PUSH0)
		&&(!(switch_raw & SAMPLE_SWITCH_BIT_PUSHALL)))
		{
			switch_detect_on &= ~SAMPLE_SWITCH_BIT_PUSH0;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			//printf("SAMPLE_SWITCH_BIT_PUSH0\n");
			break;	// Exit Push-Switch 0 and 1 becomes ON.
		}
		// Push-Switch 1
		if( (switch_detect_on & SAMPLE_SWITCH_BIT_PUSH1)
		&&(!(switch_raw & SAMPLE_SWITCH_BIT_PUSHALL)))
		{
			switch_detect_on &= ~SAMPLE_SWITCH_BIT_PUSH1;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			//printf("SAMPLE_SWITCH_BIT_PUSH1\n");
			printf("For the tests to customize TTB.\n");
			sample_mmu_test_ttb_desc_set_APIs();
			disp_usage = true;
		}
		// Push-Switch 2
		if( (switch_detect_on & SAMPLE_SWITCH_BIT_PUSH2)
		&&(!(switch_raw & SAMPLE_SWITCH_BIT_PUSHALL)))
		{
			switch_detect_on &= ~SAMPLE_SWITCH_BIT_PUSH2;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			//printf("SAMPLE_SWITCH_BIT_PUSH2\n");
			printf("Invalidate the entire unified TLB.\n");
			sample_mmu_test_TLB_maintenance_operations_APIs();
			disp_usage = true;
		}
		// Push-Switch 3
		if( (switch_detect_on & SAMPLE_SWITCH_BIT_PUSH3)
		&&(!(switch_raw & SAMPLE_SWITCH_BIT_PUSHALL)))
		{
			switch_detect_on &= ~SAMPLE_SWITCH_BIT_PUSH3;

			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			//printf("SAMPLE_SWITCH_BIT_PUSH3\n");
			sample_mmu_test_print_registers();
			disp_usage = true;
		}
		
		util_intlog_print();

		switch_raw_bk = switch_raw;
	}
	printf("==== End While(1) loop process. ================\n");

	util_intlog_print();

	printf("MMU Disable.\n");
	alt_mmu_disable();

	return;
}

// All APIs to be tested in this sample.
/* MMU Management API */
/* +-- MMU Management */
/* Usage:	ALT_STATUS_CODE  alt_mmu_init (void);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_uninit (void);
	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_ttb1_init (uint32_t *ttb1);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_ttb1_desc_set (uint32_t *ttb1, const void *va, const uint32_t desc);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_ttb2_desc_set (const uint32_t *ttb1, const void *va, const uint32_t desc);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_disable (void);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_enable (void);	*/
/* Usage:	void *  alt_mmu_TTBR0_get (void);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_TTBR0_set (const void *addr);	*/
/* Usage:	void *  alt_mmu_TTBR1_get (void);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_TTBR1_set (const void *addr);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_TTBCR_set (const bool enable_ttbr0_walk, const bool enable_ttbr1_walk, const uint32_t base_addr_width);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_DACR_set (const ALT_MMU_DAP_t domain_ap[], const size_t num_elem);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_CONTEXTIDR_set (const uint32_t procid, const uint32_t asid);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_tlb_invalidate (void);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_tlb_invalidate_is (void);	*/
/* +-- MMU Virtual Address Space Creation */
/* Usage:	size_t  alt_mmu_va_space_storage_required (const ALT_MMU_MEM_REGION_t *mem_regions, const size_t num_mem_regions);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_va_space_create (uint32_t **ttb1, const ALT_MMU_MEM_REGION_t *mem_regions, const size_t num_mem_regions, alt_mmu_ttb_alloc_t ttb_alloc, void *ttb_alloc_context);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_va_space_enable (const uint32_t *ttb1);	*/

////////////////////////////// Common Functions //////////////////////////////////////// 
/********************************************************************************//*!
 * @fn  static void sample_dmac_setting_testdata(uint32_t* start, uint32_t* endnext)
 * @brief  Writes the data for transfer test.
 *
 * @details  Writes the address value area of memory that is specified for the transfer test.
 * @param[in] start  start address
 * @param[in] endnext  Next to the end address
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  新規作成.
 **//*******************************************************************************/
static void sample_dmac_setting_testdata(uint32_t* start, uint32_t* endnext)
{
	while(start < endnext){
		*start = (uint32_t)start;
		start++;
		if(start < (uint32_t*)0x00000004)	break;
	}

	return;
}


/************************************/
/*	Test Command (MMU Management)	*/
/************************************/
int sample_mmu_test_cmd(char* options)
{
    ALT_STATUS_CODE status = ALT_E_SUCCESS;
    
    printf("\r\nUser Application Start!\r\n");

    //
    // Start the interrupt system
    //
    if (status == ALT_E_SUCCESS)
    {
        status = socfpga_int_start();
    }

    // Printing Current PLL Setting. And Set Global Timer for Time Measurement.
    util_time_init();

    //// Sample Function Call!!! |---->
    printf(">>>> Execute sample_mmu_test_init(); !!! <<<<\r\n");
    sample_mmu_test_init();

    printf(">>>> Execute sample_mmu_test_main(); !!! <<<<\r\n");
    sample_mmu_test_main();

    printf(">>>> Execute sample_mmu_test_uninit(); !!! <<<<\r\n");
    sample_mmu_test_uninit();
    //// Sample Function Call!!! <----|

    // Printing All Results of Time Measurement. And Uninit Global Timer.
    util_time_uninit();

    //
    // Stop the interrupt system
    //
    socfpga_int_stop();

    printf("Finished running the sample !!!\r\n");
    if (status == ALT_E_SUCCESS)
    {
        return 0;
    }
    else
    {
        return 1;
    }
}
/***********************************************************************************
 * end of file 
 ***********************************************************************************/
