/********************************************************************************//*!
 * @file  minimal_io_armcc.c
 * @brief  Minimal IO support utility for ARMCC.
 * 
 * @details  Minimal IO support utility for ARMCC.
 * 
 * @note  nothing. 
 * 
 * @attention 
 * Copyright (C) 2013-2019 MACNICA,Inc. All Rights Reserved.\n
 *   This software is licensed "AS IS". 
 *   Please perform use of this software by a user's own responsibility and expense.
 *   It cannot guarantee in the maker side about the damage which occurred by the ab-
 *   ility not to use or use this software, and all damage that occurred secondarily.
 **//*******************************************************************************/

//#include <string.h>

#include <rt_sys.h>
#include <inttypes.h>

//#include <socal/alt_uart.h>
//#include <socal/hps.h>
//#include <socal/socal.h>

#include <alt_uart.h>
#include <hps.h>
#include <socal.h>

#define STDIN_FILENO  0
#define STDOUT_FILENO 1

//Select Line Feed Code
#define CR_LF_MODE 1

//Please Select UART ch for your system
#ifndef UART_CH
#define UART_CH (0) // 0 or 1
#endif
#if UART_CH
#define UART_LSR_ADDR         (ALT_UART1_LSR_ADDR)
#define UART_RBR_THR_DLL_ADDR (ALT_UART1_RBR_THR_DLL_ADDR)
#else
#define UART_LSR_ADDR         (ALT_UART0_LSR_ADDR)
#define UART_RBR_THR_DLL_ADDR (ALT_UART0_RBR_THR_DLL_ADDR)
#endif


/*
 * ̖O̓Cu
 * stdinAstdoutA stderr ɑ΂ĊJꂽt@CƂĎgp܂B
 * t@CnhԂ悤 _sys_open() `ƁA
 * ̓ftHgl̂܂ܕێ܂B
 */
const char __stdin_name[]  =  ":tt";
const char __stdout_name[] =  ":tt";
const char __stderr_name[] =  ":tt";

FILEHANDLE _sys_open(const char *name, int openmode)
{
	return 1; /* ׂēo͂ɂȂ܂ */
}

int _sys_close(FILEHANDLE fh)
{
	return 0;
}

int _sys_write(FILEHANDLE fh, const unsigned char *buf, unsigned len, int mode)
{
#if CR_LF_MODE
	/* Print each character to UART */
	for(int i=0; i<len; i++)
	{
		/* Wait until THR is empty */
		while(1 != ALT_UART_LSR_THRE_GET(alt_read_word(UART_LSR_ADDR)));

		/* Insert '\r' if detect '\r' */
		if(buf[i] == '\n'){
			alt_write_word(UART_RBR_THR_DLL_ADDR, '\r');
			/* Wait until THR is empty */
			while(1 != ALT_UART_LSR_THRE_GET(alt_read_word(UART_LSR_ADDR)));
		}

		/* Write character to THR */
		alt_write_word(UART_RBR_THR_DLL_ADDR, buf[i]);
	}
#else
	/* Print each character to UART */
	for(int i=0; i<len; i++)
	{
		/* Wait until THR is empty */
		while(1 != ALT_UART_LSR_THRE_GET(alt_read_word(UART_LSR_ADDR)));

		/* Write character to THR */
		alt_write_word(UART_RBR_THR_DLL_ADDR, buf[i]);
	}
#endif

	return 0;
}

int _sys_read(FILEHANDLE fh, unsigned char *buf, unsigned len, int mode)
{
	char ch;
	int index = 0;

	while(1){
	    /* Wait until DR is ready */
	    while(1 != ALT_UART_LSR_DR_GET(alt_read_word(UART_LSR_ADDR)));

	    /* Read RX Data */
	    ch = alt_read_byte(UART_RBR_THR_DLL_ADDR);

		if(ch == '\b'){
			if(index > 0){
				buf[index--] = '\0';
		    	/* Echo back */
		    	while(1 != ALT_UART_LSR_THRE_GET(alt_read_word(UART_LSR_ADDR)));
				alt_write_word(UART_RBR_THR_DLL_ADDR, '\b');
		    	while(1 != ALT_UART_LSR_THRE_GET(alt_read_word(UART_LSR_ADDR)));
				alt_write_word(UART_RBR_THR_DLL_ADDR, ' ');
		    	while(1 != ALT_UART_LSR_THRE_GET(alt_read_word(UART_LSR_ADDR)));
				alt_write_word(UART_RBR_THR_DLL_ADDR, '\b');
			}
		} else if((ch == '\r')||(ch == '\n')){
			if(ch == '\r'){
				buf[index++] = ch;
				ch = '\n';
				/* Echo back */
				while(1 != ALT_UART_LSR_THRE_GET(alt_read_word(UART_LSR_ADDR)));
				alt_write_word(UART_RBR_THR_DLL_ADDR, '\r');
			}
			buf[index++] = ch;
			/* Echo back */
			while(1 != ALT_UART_LSR_THRE_GET(alt_read_word(UART_LSR_ADDR)));
			alt_write_word(UART_RBR_THR_DLL_ADDR, '\n');

			return (len - index);		/* input finish */

		} else {
			if(ch == '\x03'){
				buf[0] = ch;
				buf[1] = '\n';
				buf[2] = '\0';

				/* Echo back */
				while(1 != ALT_UART_LSR_THRE_GET(alt_read_word(UART_LSR_ADDR)));
				alt_write_word(UART_RBR_THR_DLL_ADDR, '\r');
				while(1 != ALT_UART_LSR_THRE_GET(alt_read_word(UART_LSR_ADDR)));
				alt_write_word(UART_RBR_THR_DLL_ADDR, '\n');

				return (len - 2);		/* input cancel */

			} else if((ch < ' ')||('~' < ch)){
				ch = ' ';
			}
			buf[index++] = ch;
			/* Echo back */
			while(1 != ALT_UART_LSR_THRE_GET(alt_read_word(UART_LSR_ADDR)));
			alt_write_word(UART_RBR_THR_DLL_ADDR, ch);
		}
		if(index >= len)	return 0;	/* input upper limit */
	}
}

void _ttywrch(int ch)
{
	/* Wait until THR is empty */
	while(1 != ALT_UART_LSR_THRE_GET(alt_read_word(UART_LSR_ADDR)));

	/* Write character to THR */
	alt_write_word(UART_RBR_THR_DLL_ADDR, ch);
}

int _sys_istty(FILEHANDLE fh)
{
	return 0; /* obt@ꂽo */
}

int _sys_seek(FILEHANDLE fh, long pos)
{
	return -1; /* not supported */
}

long _sys_flen(FILEHANDLE fh)
{
	return -1; /* not supported */
}
